########################### PSQF 6270 Example 4b using R #################################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

#if (!require("expss")) install.packages("expss")
#library(expss) # To add variable and value labels, sorting

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("quantreg")) install.packages("quantreg")
library(quantreg) # To fit quantile regression models


######################################################################################
#####                  BEGIN DATA MANIPULATION OF EXAMPLE 4b                     #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/24_PSQF6270/PSQF6270_Example4b/"
filename = "Houses.xlsx"
setwd(dir=filesave)

# Import Houses XLSX data
Example4b = read_excel(paste0(filesave,filename))
# Convert to data frame without labels to use for analysis
Example4b = as.data.frame(Example4b)

# Categories for number of bedrooms
Example4b$bed3v2=NA; Example4b$bed3v4=NA   
Example4b$bed3v2[which(Example4b$beds==2)]=1 
Example4b$bed3v4[which(Example4b$beds==2)]=0
Example4b$bed3v2[which(Example4b$beds==3)]=0 
Example4b$bed3v4[which(Example4b$beds==3)]=0
Example4b$bed3v2[which(Example4b$beds==4)]=0 
Example4b$bed3v4[which(Example4b$beds==4)]=1
Example4b$bed3v2[which(Example4b$beds==5)]=0 
Example4b$bed3v4[which(Example4b$beds==5)]=1
# Categories for number of bathrooms
Example4b$bath2v1=NA; Example4b$bath2v3=NA   
Example4b$bath2v1[which(Example4b$baths==1)]=1 
Example4b$bath2v3[which(Example4b$baths==1)]=0
Example4b$bath2v1[which(Example4b$baths==2)]=0 
Example4b$bath2v3[which(Example4b$baths==2)]=0
Example4b$bath2v1[which(Example4b$baths==3)]=0 
Example4b$bath2v3[which(Example4b$baths==3)]=1
Example4b$bath2v1[which(Example4b$baths==4)]=0 
Example4b$bath2v3[which(Example4b$baths==4)]=1
# Center and rescale size into per 100 square feet (0=1500)
Example4b$sqft150=(Example4b$size-1500)/100
# Make squared version for use
Example4b$sqftsq=Example4b$sqft150^2
# Log-transform price for lognormal model
Example4b$logprice=log(Example4b$price)

# Label existing and new variables
#price=    "price: Sale Price in 100,000 units"
#new=      "new: House is new construction (0=no, 1=yes)"
#bed3v2=   "bed3v2: 2 bedrooms instead of 3 (0=no, 1=yes)"
#bed3v4=   "bed3v4: 4 bedrooms instead of 3 (0=no, 1=yes)"  
#bath2v1=  "bath2v1: 1 bathroom instead of 2 (0=no, 1=yes)"
#bath2v3=  "bath2v3: 3 bathrooms instead of 2 (0=no, 1=yes)"
#sqft150=  "sqft150: Square Footage per 100 feet (0=1500)"
#sqftsq=   "sqftsq: Squared Square Footage per 100 feet (0=1500)"
#logprice= "logprice: Natural log of sale price in 100,000 units"


######################################################################################
#####                   BEGIN EXAMPLE 4b CONTINUOUS MODELS                       #####
######################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example4b_R_Output.txt"))

# to save a plot: open a file, create the plot, then close the file
png(file = "R Price Histogram.png")  # open file
hist(x=Example4b$price, freq=FALSE,
     ylab="Density",xlab="Sale Price in 100,000 units") # axis labels
dev.off()  # close file
png(file = "R Price Boxplot.png")  # open file
boxplot(x=Example4b$price)
dev.off()  # close file

print("R Descriptive Stats for Example Variables")
describe(x=Example4b$price); describe(x=Example4b$size)
table(x=Example4b$beds,useNA="ifany")
table(x=Example4b$baths,useNA="ifany")
table(x=Example4b$new,useNA="ifany")

txtComment(" ") # insert blank space in output 

############################################################################################
print("R Predict Price using Identity Link, Normal Distribution")
ModelNorm = glm(data=Example4b, family=gaussian(link="identity"), # I(x^2) squares predictor
                formula=price~1+new+bed3v2+bed3v4+bath2v1+bath2v3+sqft150+sqftsq)
print("Print -2LL with results"); -2*logLik(ModelNorm); summary(ModelNorm)

print("Multiv Wald Test of Model")
NormR2 = glht(model=ModelNorm, linfct=c("new=0","bed3v2=0","bed3v4=0","bath2v1=0",
              "bath2v3=0","sqft150=0","sqftsq=0"))  # Couldn't square predictor here
summary(NormR2, test=Chisqtest()) # Joint chi-square test 

txtComment(" ") # insert blank space in output 

############################################################################################
print("R Predict Log-Transformed Price using Identity Link, Normal Distribution")
ModelLogNorm = glm(data=Example4b, family=gaussian(link="identity"), 
                   formula=logprice~1+new+bed3v2+bed3v4+bath2v1+bath2v3+sqft150+sqftsq)
print("Print -2LL with results"); -2*logLik(ModelLogNorm); summary(ModelLogNorm)

print("Multiv Wald Test of Model -- matches SAS using REML")
LogTNormR2 = glht(model=ModelLogNorm, linfct=c("new=0","bed3v2=0","bed3v4=0",
                  "bath2v1=0","bath2v3=0","sqft150=0","sqftsq=0"))
summary(LogTNormR2, test=Chisqtest()) # Joint chi-square test 

# Could not find lognormal conditional distribution that was likelihood-equivalent

txtComment(" ") # insert blank space in output 

############################################################################################
print("R Predict Price using Log Link, Gamma Distribution")
print("SEs and scale parameter differ slightly from SAS and STATA")
ModelGamma = glm(data=Example4b, family=Gamma(link="log"), # I(x^2) squares predictor
                 formula=price~1+new+bed3v2+bed3v4+bath2v1+bath2v3+sqft150+sqftsq)
print("Print -2LL, with results"); -2*logLik(ModelGamma); summary(ModelGamma)

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelGamma, type="pearson")^2)/(100-8)

print("Multiv Wald Test of Model -- differs from SAS and STATA")
GammaR2 = glht(model=ModelGamma, linfct=c("new=0","bed3v2=0","bed3v4=0",
               "bath2v1=0","bath2v3=0","sqft150=0","sqftsq=0"))
summary(GammaR2, test=Chisqtest()) # Joint chi-square test 

print("Get incidence rate ratios with 95% CIs")
exp(cbind(IRR=coefficients(ModelGamma), confint.default(ModelGamma)))

txtComment(" ") # insert blank space in output 

############################################################################################
print("R Predict Price 50th Percentile [Median] using Quantile Regression")
print("Did not figure out how to get same SEs and test statistics as SAS and STATA")
set.seed(8675309) # Jenny is my random seed
ModelQ50 = rq(data=Example4b, tau=.5,
              formula=price~1+new+bed3v2+bed3v4+bath2v1+bath2v3+sqft150+sqftsq)
summary(ModelQ50)

txtComment(" ") # insert blank space in output 

print("R Predict Price 25th and 75th Percentile using Quantile Regression")
print("Did not figure out how to get same SEs and test statistics as SAS and STATA")
set.seed(8675309) # Jenny is my random seed
ModelQ2575 = rq(data=Example4b, tau=c(.25,.75),
                formula=price~1+new+bed3v2+bed3v4+bath2v1+bath2v3+sqft150+sqftsq)
summary(ModelQ2575)

txtComment(" ") # insert blank space in output 

# I could not find how to fit all quantiles simultaneously in R, so I gave up

############################################################################################
# Close external output file
txtStop()




